/*********************************************************************
*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*           (C) 1996    SEGGER Microcontroller Systeme GmbH          *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File        : GUI_X.C
Purpose     : Config / System dependent externals for GUI
---------------------------END-OF-HEADER------------------------------
*/
#include <stdio.h>
#include "GUI.h"
#include "LPC17XX.H"
GUI_PID_STATE tc;
/*********************************************************************
*
*       Global data
*/
volatile int TimeMS;

/*********************************************************************
*
*      Timing:
*                 GUI_X_GetTime()
*                 GUI_X_Delay(int)

  Some timing dependent routines require a GetTime
  and delay function. Default time unit (tick), normally is
1 ms.
*/

int GUI_X_GetTime(void) {
  return TimeMS;
}

void GUI_X_Delay(int ms) {
  int tEnd = TimeMS + ms;
  while ((tEnd - TimeMS) > 0);
}

/*********************************************************************
*
*       GUI_X_Init()
*
* Note:
*     GUI_X_Init() is called from GUI_Init is a possibility to init
*     some hardware which needs to be up and running before the GUI.
*     If not required, leave this routine blank.
*/

void GUI_X_Init(void) {
  TimeMS = 0;
}


/*********************************************************************
*
*       GUI_X_ExecIdle
*
* Note:
*  Called if WM is in idle state
*/

void GUI_X_ExecIdle(void) {}

/*********************************************************************
*
*      Multitasking:
*
*                 GUI_X_InitOS()
*                 GUI_X_GetTaskId()
*                 GUI_X_Lock()
*                 GUI_X_Unlock()
*
* Note:
*   The following routines are required only if emWin is used in a
*   true multi task environment, which means you have more than one
*   thread using the emWin API.
*   In this case the
*                       #define GUI_OS 1
*  needs to be in GUIConf.h
*/


//static OS_RSEMA RSema;

void GUI_X_InitOS(void)    { /*OS_CreateRSema(&RSema);*/    }
void GUI_X_Unlock(void)    { /*OS_Unuse(&RSema);*/ }
void GUI_X_Lock(void)      { /*OS_Use(&RSema);*/  }
U32  GUI_X_GetTaskId(void) { return 0; /*(U32)OS_GetTaskID();*/ }

/*********************************************************************
*
*      Logging: OS dependent

Note:
  Logging is used in higher debug levels only. The typical target
  build does not use logging and does therefor not require any of
  the logging routines below. For a release build without logging
  the routines below may be eliminated to save some space.
  (If the linker is not function aware and eliminates unreferenced
  functions automatically)

*/

void GUI_X_Log     (const char *s) { GUI_USE_PARA(s); }
void GUI_X_Warn    (const char *s) { GUI_USE_PARA(s); }
void GUI_X_ErrorOut(const char *s) { GUI_USE_PARA(s); }
void Del(void){
	int i=50000;
	while(i--);
}
/*************************** End of file ****************************/
int  GUI_TOUCH_X_MeasureX(void) {
int b;
	LPC_GPIO0->FIODIR |= (1<<23)|(1<<25) ;
	LPC_PINCON->PINMODE1 |= (1UL<<17)|(1UL<<21);
	LPC_GPIO0->FIOSET |= (1<<25) ;
	LPC_GPIO0->FIOCLR |= (1<<23) ;
	Del();
	b= ADC(1);
	GUI_TOUCH_GetState(&tc);
  return b;
}

int  GUI_TOUCH_X_MeasureY(void) {
	int a;
	
	LPC_PINCON->PINSEL1 &= ~(1<<16) ;
	LPC_PINCON->PINMODE1 &= ~((1UL<<17)|(1UL<<21));
	LPC_GPIO0->FIODIR &= ~((1<<23)|(1<<25)) ;
	LPC_GPIO0->FIOSET &= ~(1<<23) ;
	LPC_GPIO0->FIOCLR &= ~((1<<25)|(1<<26)) ;
	LPC_GPIO0->FIODIR |= (1<<24)|(1<<26) ;
	LPC_PINCON->PINMODE1 |= (1UL<<15)|(1UL<<19);
	LPC_GPIO0->FIOSET |= (1<<24) ;
	LPC_GPIO0->FIOCLR |= (1<<26) ;
	Del();
	a= ADC(2);
	GUI_TOUCH_GetState(&tc);
return a	;
}